----------------------------------------------------------
--Joystick Manipulator
--Version 0.1
--Started: 12/24/00
--Edited:  12/25/00
--by Borislav Petrov [Boboland]
--http://gfxcentral.com/bobo/
--Idea by Mitch Gates
----------------------------------------------------------

plugin simpleManipulator joystickManipulator
    name:"Joystick"
    classID:#(0xb779962b, 0x73866712)
    category:"Manipulators"
(

	local old_y = 0.0
	
    parameters main rollout:paramRollout
    ( 
         -- The value of the slider
         valueX      type:#float   animatable:true  ui:spnValueX  default: 0.0
         valueY      type:#float   animatable:true  ui:spnValueY  default: 0.0


         -- The min and max values the slider can take
         minValX       type:#float   animatable:true  ui:spnMinX      default: -100.0
         maxValX       type:#float   animatable:true  ui:spnMaxX      default: 100.0
         defValX       type:#float   animatable:true  ui:spnDefX      default: 0.0
         minValY       type:#float   animatable:true  ui:spnMinY      default: -100.0
         maxValY       type:#float   animatable:true  ui:spnMaxY      default: 100.0
         defValY       type:#float   animatable:true  ui:spnDefY      default: 0.0
		 
         -- The slider position as a percentage of screen space
         xPos       type:#float   animatable:false ui:spnXPos   default: 0.0
         yPos       type:#float   animatable:false ui:spnYPos   default: 0.0
		 
		 size 		type:#float		animatble:false ui:spnSize	default: 25.0
         -- When the hide flag is true, the slider bar and value are not draw
         hide       type:#boolean animatable:false ui:checkHide default: false

         -- The display name of the slider
         sldName    type:#string  animatable:false              default: ""

         -- increment value
         autoCenter type:#boolean   animatable:false  ui:checkautoCenter   default: true
		 
		 -- user colors
		 UserGreenColor		type:#color animatable:false	ui:UserGreenColor	default:(color 0 255 0)
		 UserYellowColor	type:#color animatable:false	ui:UserYellowColor	default:(color 255 255 0)
		 UserRedColor		type:#color animatable:false	ui:UserRedColor	default:(color 255 0 0)
		 UseCustomColors	type:#boolean animatable:false ui:UseCustomColors default: false
    )

    -- The rollout for the slider parameters
    rollout paramRollout "Joystick Manipulator"
    (
		Group "Joystick Parameters"
		(
		Label sliderName "Name:" across:2 align:#left
        Checkbox checkHide  "Hide"         checked:false align:#right
        Edittext nameEdit   ""        text:""	height:32 offset:[-5,0]
        Spinner  spnValueX   "X Value:"       range:[-1.0e30, 1.0e30, 0.0] 
        Spinner  spnValueY   "Y Value:"       range:[-1.0e30, 1.0e30, 0.0] 

        Spinner  spnMinX     "Minimum X:"     range:[-1.0e30, 0, -100.0] 
        Spinner  spnMinY     "Minimum Y:"     range:[-1.0e30, 0, -100.0] 
		
        Spinner  spnDefX     "Default X:"     range:[-1.0e30, 1.0e30, 0.0] 
        Spinner  spnDefY     "Default Y:"     range:[-1.0e30, 1.0e30, 0.0] 

        Spinner  spnMaxX     "Maximum X:"     range:[0, 1.0e30, 100.0] 
        Spinner  spnMaxY     "Maximum Y:"     range:[0, 1.0e30, 100.0] 
		
		)
		
		Group "Position and Size"
		(
        Spinner  spnXPos    "X Position:"  range:[0.0, 1.0, 0.0] scale:0.01
        Spinner  spnYPos    "Y Position:"  range:[0.0, 1.0, 0.0] scale:0.01
        Spinner  spnSize    "Size:"  range:[10.0, 100.0, 15.0] 
		)
		
        Checkbutton checkAutoCenter "Auto-Center" width:140
		
		fn filter_sliders obj = ( FindItem (getPropNames obj) #UserGreenColor > 0 )	
		
		Group "Custom Colors"
		(
		checkbox UseCustomColors "Use" align:#left across:2
		ColorPicker UserGreenColor	"Active"		align:#right fieldwidth:40
		ColorPicker UserYellowColor	"Inactive"		align:#right fieldwidth:40
		ColorPicker UserRedColor	"Selected"		align:#right fieldwidth:40
		PickButton getColors "Acquire Colors From..." width:140 filter:filter_sliders
		)
		
		On getColors picked obj do
		(
		UserGreenColor.color = obj.UserGreenColor
		UserYellowColor.color = obj.UserYellowColor
		UserRedColor.color = obj.UserRedColor
		UseCustomColors.checked = obj.UseCustomColors
		)
		
        on nameEdit entered txt do 
        (
           sldName = txt
        )

        on paramRollout open do 
        (
           nameEdit.text = sldName
        )
    )

    -- This manipualtor manipulates itself
    on canManipulate target  return (classOf target) == joystickManipulator

    -- Creation mouse procedure
    tool create
    (
        on mousePoint click do
           case click of
           (
              1: 
              (
                -- Compute the percentage of of the mouse point in screen space
                this.xPos = viewPoint.x / gw.getWinSizeX()
                this.yPos = viewPoint.y / gw.getWinSizeY()

                #stop 
              )
           )

    )

    -- Create the manipulator gizmos.
    on updateGizmos do
    (
        -- Clear the current gizmo cache
        this.clearGizmos()

        -- If this is not a stand-alone manip, get values from the manip target
        if (target != undefined) then 
        (
           this.valueX      = target.valueX
           this.valueY      = target.valueY
		   
           this.minValX     = target.minValX
           this.maxValX     = target.maxValX
           this.minValY     = target.minValY
           this.maxValY     = target.maxValY
           this.defValX    = target.defValX
           this.defValY    = target.defValY
		   
           this.xPos       = target.xPos
           this.yPos       = target.yPos
	       this.hide       = target.hide
           this.sldName    = target.sldName
           this.size 	   = target.size
           this.autoCenter = target.autoCenter
		   
		   if target.UseCustomColors then
		   (
		   UGreenColor   =  target.UserGreenColor / 255.0
		   URedColor =    target.UserRedColor / 255.0
		   )
		   else
		   (
    		UGreenColor   =  colorMan.getColor #manipulatorsActive   
		    URedColor =    colorMan.getColor #manipulatorsSelected
		   )
        ) 
        else
        (
		   if this.UseCustomColors then
			(
			UGreenColor   =  this.UserYellowColor / 255.0
		    URedColor =    this.UserRedColor / 255.0
			)
			else
			(
    		UGreenColor   =  colorMan.getColor #manipulatorsInactive
		    URedColor =    colorMan.getColor #manipulatorsSelected
			)
        )


        -- Compute the origin of the slider in screen space
        local x = xPos * gw.getWinSizeX()
        local y = yPos * gw.getWinSizeY()

        -- These flags are used for all the gizmos
        local flags = (gizmoUseScreenSpace + gizmoActiveViewportOnly)

        -- Gizmo 0
        -- This is the box that you use the move the manipulator
        
        this.addGizmoMarker #hollowBox [x+Size+10, y, 0.0] flags UGreenColor URedColor

		
        -- Gizmo 1
        -- This is the plus-sign used to toggle the hide flag
		
        this.addGizmoMarker #plusSign [x+Size+20,y,0] flags UGreenColor  URedColor

        -- Compute the text label
        if hide then 
		(
			str1 = sldName
			str2 = ""
		)
        else 
        (
            if sldName.count > 0 then 
			(
				str1 = sldName 
				str2 = "[" + (valueX as string) + "," +(valueY as string)+"]"
			)
            else 
			(
				str1 = ""
				str2 = "[" + (valueX as string) + "," +(valueY as string)+"]"
			)	
        )

        -- Gizmo 2 and 3
        -- The text with the name and value
        this.addGizmoText str1 [x+15+size, y-4 , 0.0] flags UGreenColor  URedColor
        this.addGizmoText str2 [x+15+size, y+20 , 0.0] flags UGreenColor  URedColor


        -- If the slider is hidden, don't draw any more
        if hide then return ""

        -- Draw spinner Gizmo 4
        giz = manip.makeGizmoShape()
        giz.AddPoint [x-size-4,y-size-4,0]
        giz.AddPoint [x+size+4,y-size-4,0]
        giz.AddPoint [x+size+4,y+size+4,0]
        giz.AddPoint [x-size-4,y+size+4,0]
        giz.AddPoint [x-size-4,y-size-4,0]
		
        this.addGizmoShape giz flags UGreenColor  UGreenColor
		
		
        -- Draw spinner Gizmo 5
		if ValueX < 0.0 then
			(
			DeltaX = (abs(ValueX)/MinValX)*Size
			)
			else
			(
			DeltaX = ValueX/MaxValX*Size
			)

		if ValueY < 0.0 then
			(
			DeltaY = (abs(ValueY)/MinValY)*Size
			)
			else
			(
			DeltaY = ValueY/MaxValY*Size
			)
			
		giz = manip.MakeCircle [x+DeltaX,y+DeltaY,0] 4 16	
        this.addGizmoShape giz flags UGreenColor  URedColor
		
		
        -- Gizmo 6
        -- The resize diamond
        this.addGizmoMarker #diamond [x+Size+7,y+Size+7,0] flags UGreenColor URedColor		
        
        -- Gizmo 7
		-- Auto-Center Toggle
		
		if autoCenter then
		(
		this.addGizmoMarker #plusSign [x+Size+7,y-Size-7,0] flags UGreenColor  URedColor		
		)
		else
		(
		this.addGizmoMarker #hollowbox [x+Size+7,y-Size-7,0] flags UGreenColor  URedColor		
		)
		
		
        -- return an empty string so no tool tip is displayed
        return ""
    )

    -- mouseMove is called on every mouse move when dragging the manip
    -- It needs to convert the mouse position 'm' into a new value for the slider
    on mouseMove m which do
    (
        local xSize = gw.getWinSizeX()
        local ySize = gw.getWinSizeY()
        if (which == 0)  then 
        (
            -- Gizmo 0
            -- Set the slider postition
            target.xPos = (m.x-Size-10) / xSize
            target.yPos = m.y / ySize
        ) 

        if (which == 5)  then 
        (
            -- Gizmo 5
            -- Set the joystick position
			NewValX = (m.x-(target.xPos*xSize)) / Size
			NewValY = (m.y-(target.yPos*ySize)) / Size
		
			if NewValX > 1.0 then NewValX = 1.0
			if NewValY > 1.0 then NewValY = 1.0
			if NewValX < -1.0 then NewValX = -1.0
			if NewValY < -1.0 then NewValY = -1.0

				
			
			if NewValX < 0.0 then 
				(
				NewValX = abs(NewValX)*MinValX
				)
				else
				(
				NewValX *= MaxValX
				)
			if NewValY < 0.0 then 
				(
				NewValY = abs(NewValY)*MinValY
				)
				else
				(
				NewValY *= MaxValY
				)
--			format "%, % \n" NewValX NewValY	
            target.ValueX = NewValX
            target.ValueY = NewValY
        ) 

        if (which == 6)  then 
        (
            -- Gizmo 6
            -- Set the size
			delta = (m.x-(target.xPos*xSize)-7.0)
			if delta > 100 then delta = 100
			if delta < 10 then delta = 10
			
            target.size = delta
        ) 

		

	)
	
    on mouseDown m which do
    (
      -- Toggle the value of the "Hide" state
      if (which == 1) then target.hide = not hide 
      if (which == 7) then 
	  	(
		target.autoCenter = not autoCenter
		if target.autoCenter then 
			(
            target.ValueX = defValX
            target.ValueY = defValY
			)
		)
	  
    )
	
    on mouseUp m which do
    (
      if (which == 5) then 
  		(
		if autoCenter then
			(
            target.ValueX = defValX
            target.ValueY = defValY
			)
		)
    )
)
